import{j as s}from"./@radix-D8tIKeo-.js";import{T as r,be as i,c as n,bf as o}from"./index-B44sgOyQ.js";function c({status:e,className:t,...a}){return s.jsxs(r,{size:"xs",emphasis:"subtle",rounded:!1,className:n("inline-flex items-center gap-1 text-theme-text-primary",t),color:"grey",...a,children:[s.jsx("div",{className:`size-1 shrink-0 rounded-rounded ${o(e)}`}),i(e??"")]})}export{c as D};
