"""Release [0.45.3].

Revision ID: 0.45.3
Revises: 729263e47b55
Create Date: 2023-10-18 14:39:09.972710

"""

# revision identifiers, used by Alembic.
revision = "0.45.3"
down_revision = "729263e47b55"
branch_labels = None
depends_on = None


def upgrade() -> None:
    """Upgrade database schema and/or data, creating a new revision."""
    pass


def downgrade() -> None:
    """Downgrade database schema and/or data back to the previous revision."""
    pass
