import{j as r}from"./@radix-D8tIKeo-.js";import{c as n,bS as i,ar as o,x as m}from"./index-B44sgOyQ.js";import{S as l}from"./alert-triangle-DTrw87FZ.js";const c=i("flex items-center text-text-sm rounded-md border px-4 py-3",{variants:{intent:{primary:"border-primary-400 bg-primary-25",warning:"bg-[#FFF6EA] border-theme-border-moderate",neutral:"border-theme-border-moderate",error:"bg-error-50 border-error-300"}},defaultVariants:{intent:"primary"}});function u({children:e,className:s,intent:a,...t}){return r.jsxs("div",{...t,className:n(c({intent:a}),s),children:[r.jsx(d,{intent:a}),r.jsx("div",{className:"w-full min-w-0",children:e})]})}function d({intent:e}){switch(e){case"warning":return r.jsx(l,{className:"mr-4 h-5 w-5 shrink-0 fill-warning-700"});case"error":return r.jsx(m,{className:"mr-4 h-5 w-5 shrink-0 fill-error-700"});default:return r.jsx(o,{className:"mr-4 h-5 w-5 shrink-0 fill-theme-text-brand"})}}export{u as I};
