import{j as t}from"./@radix-D8tIKeo-.js";import{C as o}from"./form-pNZeTxi1.js";import{a5 as e,bc as s,r as a}from"./index-B44sgOyQ.js";import{a as m}from"./@monaco-editor-CuNQBT0l.js";import"./@elkjs-CmhuJAjY.js";import"./@zod-2qqdh1cM.js";import"./@react-router-D1ODL_Ub.js";import"./name-Bg7MBXU_.js";import"./@tanstack-nMqraTcu.js";import"./all-pipeline-runs-query-g0Sk3bVP.js";import"./pipeline-run-detail-query-BeIs5CHr.js";import"./update-snapshot-DClhy_Oa.js";import"./@reactflow-DgowYF6H.js";function p(){const{setBreadcrumbs:r}=e();m.useEffect(()=>{r([s,{label:"Create",href:a.projects.snapshots.create}])},[r])}function E(){return p(),t.jsx("div",{className:"mx-auto w-full max-w-4xl p-5",children:t.jsx(o,{run:void 0})})}export{E as default};
