import{j as e}from"./@radix-D8tIKeo-.js";import{c as s,a,b as n,d as p,e as i,u as l,D as u,P as c,f as y}from"./column-definitions-CZ9HzQPs.js";import{a as D}from"./@monaco-editor-CuNQBT0l.js";import{a as d}from"./@react-router-D1ODL_Ub.js";import"./@elkjs-CmhuJAjY.js";import"./Pagination-Bpjzny1d.js";import"./index-B44sgOyQ.js";import"./@tanstack-nMqraTcu.js";import"./@reactflow-DgowYF6H.js";import"./@zod-2qqdh1cM.js";import"./chevron-right-double-DLhLU36B.js";import"./refresh-DNxZUtB4.js";import"./helper-Bk0vl-dI.js";import"./SearchField-BOlpKJtF.js";import"./button-group-basic-D6kW-U5o.js";import"./trash-Dp8wQrEE.js";import"./DeleteAlertDialog-DbZA_NGz.js";import"./deployment-status-tag-DFSMQzTa.js";import"./pipeline-link-D3UQVHWj.js";function x(){return D.useMemo(()=>[s(),a(),n(),p(),i()],[])}function P(){const{pipelineId:o}=d(),r=l(),m=x(),t={...r,pipeline:o,sort_by:"desc:updated"};return e.jsxs(e.Fragment,{children:[e.jsx(u,{params:t}),e.jsx(c,{params:t,columns:m})]})}function q(){return e.jsx("div",{className:"space-y-5",children:e.jsx(y,{children:e.jsx(P,{})})})}export{q as default};
