import{j as r}from"./@radix-D8tIKeo-.js";import{R as i}from"./ButtonGroup-1QXCEfkt.js";import{u as s,R as p,P as n,g as a}from"./service-DASKXYmV.js";import{a as l}from"./@react-router-D1ODL_Ub.js";import"./@monaco-editor-CuNQBT0l.js";import"./@elkjs-CmhuJAjY.js";import"./trash-Dp8wQrEE.js";import"./DeleteAlertDialog-DbZA_NGz.js";import"./@zod-2qqdh1cM.js";import"./index-B44sgOyQ.js";import"./@tanstack-nMqraTcu.js";import"./@reactflow-DgowYF6H.js";import"./helper-Bk0vl-dI.js";import"./delete-run-ApKnxGry.js";import"./bulk-delete-0rjuYwfx.js";import"./action-cell-BgyPfYXo.js";import"./schedule-tag-FF3JaJ1t.js";import"./clock-COypnJ_0.js";import"./refresh-DNxZUtB4.js";import"./SearchField-BOlpKJtF.js";import"./all-pipeline-runs-query-g0Sk3bVP.js";import"./Pagination-Bpjzny1d.js";import"./chevron-right-double-DLhLU36B.js";function u(){const o=s(),m=a(),{deploymentId:e}=l(),t={triggered_by_deployment_id:e,...o};return r.jsxs(r.Fragment,{children:[r.jsx(p,{params:t}),r.jsx(n,{columns:m,params:t})]})}function w(){return r.jsx("div",{className:"space-y-5 p-5 lg:px-[80px]",children:r.jsx(i,{children:r.jsx(u,{})})})}export{w as default};
