import{a6 as a,E as u,aT as i,G as o,F as c,H as p}from"./index-B44sgOyQ.js";import{b as f}from"./@tanstack-nMqraTcu.js";function h({runId:e,queryParams:t={}}){return["runs",e,t]}async function l({runId:e,queryParams:t={}}){const n=a(t),r=c(p.runs.detail(e))+(n?`?${n}`:""),s=await u(r,{method:"GET",headers:{"Content-Type":"application/json"}});if(s.status===404&&i(),!s.ok)throw new o({message:`Error while fetching pipeline run ${e}`,status:s.status,statusText:s.statusText});return s.json()}function P(e,t){return f({queryKey:h(e),queryFn:()=>l(e),...t})}export{l as f,P as u};
