import{j as e}from"./@radix-D8tIKeo-.js";import{x as s,b as a}from"./index-B44sgOyQ.js";import{E as l}from"./EmptyState-CGTtSpJ6.js";function c({err:t,isAlertCircle:r=!1}){return e.jsx(l,{icon:r?e.jsx(s,{className:"h-[120px] w-[120px] fill-neutral-300"}):e.jsx(a,{className:"h-[120px] w-[120px] fill-neutral-300"}),children:e.jsx("div",{className:"text-center",children:e.jsx("p",{className:"max-w-full whitespace-pre-wrap break-words text-text-lg text-theme-text-secondary",children:t.message})})})}export{c as E};
