import{j as e}from"./@radix-D8tIKeo-.js";import{r as l,T as p,c as s,W as o}from"./index-B44sgOyQ.js";import{L as m}from"./@react-router-D1ODL_Ub.js";function d({pipelineId:i,pipelineName:r,iconClassname:a,className:n,...t}){return e.jsx(m,{to:l.projects.pipelines.detail.runs(i),children:e.jsxs(p,{color:"purple",className:s("inline-flex items-center gap-0.5 text-primary-400",n),rounded:!1,emphasis:"subtle",...t,children:[e.jsx(o,{className:s("size-4 shrink-0 fill-primary-400",a)}),e.jsx("div",{className:"truncate",children:r})]})})}export{d as P};
