import{j as r}from"./@radix-D8tIKeo-.js";import{R as m}from"./ButtonGroup-1QXCEfkt.js";import{a5 as a}from"./index-B44sgOyQ.js";import{a as s}from"./@monaco-editor-CuNQBT0l.js";import{u as p,g as n,R as u,P as c}from"./service-DASKXYmV.js";import{a as l}from"./@react-router-D1ODL_Ub.js";import"./@elkjs-CmhuJAjY.js";import"./trash-Dp8wQrEE.js";import"./DeleteAlertDialog-DbZA_NGz.js";import"./@zod-2qqdh1cM.js";import"./helper-Bk0vl-dI.js";import"./delete-run-ApKnxGry.js";import"./@tanstack-nMqraTcu.js";import"./bulk-delete-0rjuYwfx.js";import"./@reactflow-DgowYF6H.js";import"./action-cell-BgyPfYXo.js";import"./schedule-tag-FF3JaJ1t.js";import"./clock-COypnJ_0.js";import"./refresh-DNxZUtB4.js";import"./SearchField-BOlpKJtF.js";import"./all-pipeline-runs-query-g0Sk3bVP.js";import"./Pagination-Bpjzny1d.js";import"./chevron-right-double-DLhLU36B.js";function d(e){const{setBreadcrumbs:t}=a();s.useEffect(()=>{},[t,e])}function x(){const{pipelineId:e}=l(),t=p(),o=s.useMemo(()=>n(),[]),i={pipeline_id:e,...t,sort_by:"desc:updated"};return r.jsxs(r.Fragment,{children:[r.jsx(u,{params:i}),r.jsx(c,{columns:o,params:i})]})}function z(){return d(),r.jsx("div",{className:"space-y-5",children:r.jsx(m,{children:r.jsx(x,{})})})}export{z as default};
