import{j as e}from"./@radix-D8tIKeo-.js";import{c as r,a as m,g as a,b as s,d as n,e as p,u as l,D as i,P as u,f as c}from"./column-definitions-CZ9HzQPs.js";import{a as y}from"./@monaco-editor-CuNQBT0l.js";import"./@elkjs-CmhuJAjY.js";import"./Pagination-Bpjzny1d.js";import"./index-B44sgOyQ.js";import"./@tanstack-nMqraTcu.js";import"./@react-router-D1ODL_Ub.js";import"./@reactflow-DgowYF6H.js";import"./@zod-2qqdh1cM.js";import"./chevron-right-double-DLhLU36B.js";import"./refresh-DNxZUtB4.js";import"./helper-Bk0vl-dI.js";import"./SearchField-BOlpKJtF.js";import"./button-group-basic-D6kW-U5o.js";import"./trash-Dp8wQrEE.js";import"./DeleteAlertDialog-DbZA_NGz.js";import"./deployment-status-tag-DFSMQzTa.js";import"./pipeline-link-D3UQVHWj.js";function D(){return y.useMemo(()=>[r(),m(),a(),s(),n(),p()],[])}function x(){const o=D(),t={...l(),sort_by:"desc:updated"};return e.jsxs(e.Fragment,{children:[e.jsx(i,{params:t}),e.jsx(u,{params:t,columns:o})]})}function _(){return e.jsx("div",{className:"space-y-5 py-5",children:e.jsx(c,{children:e.jsx(x,{})})})}export{_ as default};
