import{j as t}from"./@radix-D8tIKeo-.js";import{C as o,S as r}from"./StackComponentList-DOJkkuHf.js";import"./@monaco-editor-CuNQBT0l.js";import"./@elkjs-CmhuJAjY.js";import"./plus-BjxzjZ9E.js";import"./refresh-DNxZUtB4.js";import"./Pagination-Bpjzny1d.js";import"./index-B44sgOyQ.js";import"./@tanstack-nMqraTcu.js";import"./@react-router-D1ODL_Ub.js";import"./@reactflow-DgowYF6H.js";import"./@zod-2qqdh1cM.js";import"./chevron-right-double-DLhLU36B.js";import"./SearchField-BOlpKJtF.js";import"./expand-full-h75BCAl0.js";import"./resizable-sheet-CvLdhybR.js";import"./react-resizable-panels.browser-0v9wo-oN.js";import"./Tabs-BQs5gdOY.js";import"./scrolling-tabs-list-PzSayIJ0.js";import"./chevron-down-C9YWkVDM.js";import"./transform-DsIiQtL0.js";import"./KeyValue-BtlwXwcP.js";import"./eye-CuzpINsY.js";import"./CollapsibleCard-B9jxOFAU.js";import"./CodeSnippet-WWMTzt1F.js";import"./all-pipeline-runs-query-g0Sk3bVP.js";import"./index-Ddinc6X-.js";import"./stack-detail-query-DvU8xhXI.js";import"./ComponentBadge-DaTkhzZ4.js";import"./ComponentIcon-Btqtj6Xx.js";import"./bar-chart-square-check-BAWR96En.js";import"./layout-BBB8hotd.js";import"./action-cell-BgyPfYXo.js";import"./trash-Dp8wQrEE.js";import"./DeleteAlertDialog-DbZA_NGz.js";import"./Infobox-CIL_nbOc.js";import"./alert-triangle-DTrw87FZ.js";import"./helper-Bk0vl-dI.js";import"./bulk-delete-0rjuYwfx.js";function M(){return t.jsx("div",{className:"pt-5",children:t.jsx(o,{children:t.jsx(r,{})})})}export{M as default};
