"""Release [0.56.1].

Revision ID: 0.56.1
Revises: 0.56.0
Create Date: 2024-03-21 14:50:20.869911

"""

# revision identifiers, used by Alembic.
revision = "0.56.1"
down_revision = "0.56.0"
branch_labels = None
depends_on = None


def upgrade() -> None:
    """Upgrade database schema and/or data, creating a new revision."""
    pass


def downgrade() -> None:
    """Downgrade database schema and/or data back to the previous revision."""
    pass
