import{j as r}from"./@radix-D8tIKeo-.js";import{ao as l,bh as i,c as e,bR as n}from"./index-B44sgOyQ.js";import{a as c}from"./@monaco-editor-CuNQBT0l.js";const m=c.forwardRef(({children:s,className:a,...o},t)=>r.jsxs(l,{children:[r.jsx(i,{ref:t,className:e("w-full flex-nowrap",a),...o,children:s}),r.jsx(n,{orientation:"horizontal"})]}));m.displayName="ScrollingTabsList";export{m as S};
