import{j as t}from"./@radix-D8tIKeo-.js";import{c as n,a as i,b as m,d as p,e as u,f as c,g as l,u as h,S as d,P as S,h as f}from"./column-definitions-BSD1hyp2.js";import{a}from"./@monaco-editor-CuNQBT0l.js";import{a as x}from"./@react-router-D1ODL_Ub.js";import{a5 as C}from"./index-B44sgOyQ.js";import"./@elkjs-CmhuJAjY.js";import"./plus-BjxzjZ9E.js";import"./refresh-DNxZUtB4.js";import"./SearchField-BOlpKJtF.js";import"./@tanstack-nMqraTcu.js";import"./button-group-basic-D6kW-U5o.js";import"./trash-Dp8wQrEE.js";import"./DeleteAlertDialog-DbZA_NGz.js";import"./@zod-2qqdh1cM.js";import"./Pagination-Bpjzny1d.js";import"./chevron-right-double-DLhLU36B.js";import"./deployment-status-tag-DFSMQzTa.js";import"./pipeline-link-D3UQVHWj.js";import"./action-cell-BgyPfYXo.js";import"./helper-Bk0vl-dI.js";import"./update-snapshot-DClhy_Oa.js";import"./bulk-delete-0rjuYwfx.js";import"./@reactflow-DgowYF6H.js";function P(){return a.useMemo(()=>[n(),i(),m(),p(),u(),c(),l()],[])}function b(){const{pipelineId:e}=x(),s=h(),r=P(),o={pipeline:e,named_only:!0,...s,sort_by:"desc:updated"};return t.jsxs(t.Fragment,{children:[t.jsx(d,{params:o}),t.jsx(S,{params:o,columns:r})]})}function j(e){const{setBreadcrumbs:s}=C();a.useEffect(()=>{},[s,e])}function K(){return j(),t.jsx("div",{className:"space-y-5",children:t.jsx(f,{children:t.jsx(b,{})})})}export{K as default};
