import{j as r}from"./@radix-D8tIKeo-.js";import{R as o}from"./RunsBody-BuWck6pn.js";import{R as t}from"./ButtonGroup-1QXCEfkt.js";import"./@monaco-editor-CuNQBT0l.js";import"./@elkjs-CmhuJAjY.js";import"./refresh-DNxZUtB4.js";import"./Pagination-Bpjzny1d.js";import"./index-B44sgOyQ.js";import"./@tanstack-nMqraTcu.js";import"./@react-router-D1ODL_Ub.js";import"./@reactflow-DgowYF6H.js";import"./@zod-2qqdh1cM.js";import"./chevron-right-double-DLhLU36B.js";import"./SearchField-BOlpKJtF.js";import"./all-pipeline-runs-query-g0Sk3bVP.js";import"./pipeline-link-D3UQVHWj.js";import"./action-cell-BgyPfYXo.js";import"./schedule-tag-FF3JaJ1t.js";import"./clock-COypnJ_0.js";import"./trash-Dp8wQrEE.js";import"./DeleteAlertDialog-DbZA_NGz.js";import"./helper-Bk0vl-dI.js";import"./delete-run-ApKnxGry.js";import"./bulk-delete-0rjuYwfx.js";function b(){return r.jsx(t,{children:r.jsx(o,{})})}export{b as default};
