import{j as s}from"./@radix-D8tIKeo-.js";import{C as t}from"./form-pNZeTxi1.js";import{u as o}from"./pipeline-run-detail-query-BeIs5CHr.js";import{a5 as i,a8 as p,r as a,a9 as m}from"./index-B44sgOyQ.js";import{a as n}from"./@monaco-editor-CuNQBT0l.js";import{a as u}from"./@react-router-D1ODL_Ub.js";import"./@elkjs-CmhuJAjY.js";import"./@zod-2qqdh1cM.js";import"./name-Bg7MBXU_.js";import"./@tanstack-nMqraTcu.js";import"./all-pipeline-runs-query-g0Sk3bVP.js";import"./update-snapshot-DClhy_Oa.js";import"./@reactflow-DgowYF6H.js";function c(r){const{setBreadcrumbs:e}=i();n.useEffect(()=>{r&&e([...r.resources?.pipeline?[p,{label:r.resources.pipeline.name||"",href:a.projects.pipelines.detail.runs(r.resources.pipeline.id)}]:[m],{label:r.name||"",href:a.projects.runs.detail(r.id)},{label:"Create Snapshot",href:a.projects.runs.createSnapshot(r.id)}])},[e,r])}function g(){const{runId:r}=u(),e=o({runId:r});return c(e.data),e.isPending?s.jsx("p",{children:"Loading..."}):e.isError?s.jsx("p",{children:"Error"}):s.jsx("div",{className:"mx-auto w-full max-w-4xl p-5",children:s.jsx(t,{run:e.data})})}export{g as default};
