import{j as t}from"./@radix-D8tIKeo-.js";import{c as s,a as r,b as e,i as n,d as m,e as p,f as i,g as l,u,S as c,P as h,h as S}from"./column-definitions-BSD1hyp2.js";import{a as d}from"./@monaco-editor-CuNQBT0l.js";import"./@elkjs-CmhuJAjY.js";import"./plus-BjxzjZ9E.js";import"./refresh-DNxZUtB4.js";import"./SearchField-BOlpKJtF.js";import"./index-B44sgOyQ.js";import"./@tanstack-nMqraTcu.js";import"./@react-router-D1ODL_Ub.js";import"./@reactflow-DgowYF6H.js";import"./@zod-2qqdh1cM.js";import"./button-group-basic-D6kW-U5o.js";import"./trash-Dp8wQrEE.js";import"./DeleteAlertDialog-DbZA_NGz.js";import"./Pagination-Bpjzny1d.js";import"./chevron-right-double-DLhLU36B.js";import"./deployment-status-tag-DFSMQzTa.js";import"./pipeline-link-D3UQVHWj.js";import"./action-cell-BgyPfYXo.js";import"./helper-Bk0vl-dI.js";import"./update-snapshot-DClhy_Oa.js";import"./bulk-delete-0rjuYwfx.js";function C(){return d.useMemo(()=>[s(),r(),e(),n(),m(),p(),i(),l()],[])}function x(){const a=C(),o={named_only:!0,...u(),sort_by:"desc:updated"};return t.jsxs(t.Fragment,{children:[t.jsx(c,{displayCreateButton:!0,params:o}),t.jsx(h,{params:o,columns:a})]})}function H(){return t.jsx("div",{className:"space-y-5 py-5",children:t.jsx(S,{children:t.jsx(x,{})})})}export{H as default};
